﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class FloatableObj : MonoBehaviour {

    private WavesSum wavesSum;
    public float WaterY { get { return wavesSum.Eval(transform.position.x, useSecondary); } }
    public bool AboveWater {  get { return !UnderWater; } }
    public bool UnderWater { get { return WaterDiff > 0.0f; } }
    public float WaterDiff { get { return WaterY - transform.position.y; } }
    public float speedY = 0.0f;
    public const float gravity = -9.8f;
    public bool useSecondary = false;
    public bool oscilateOnWater = true;
    public float normalRotationSmooth = 1.0f;
    public float rotationOffset = -45.0f;
    public event Action<float> OnHitWater = delegate { };

    public void Start()
    {
        wavesSum = GameObject.FindGameObjectWithTag("WavesSum").GetComponent<WavesSum>();
    }

    public void Update()
    {
        if(oscilateOnWater)
        {
            if (AboveWater)
            {
                speedY += gravity * Time.deltaTime;
            }
            else if (UnderWater)
            {
                float sampleBehind = wavesSum.Eval(transform.position.x - 0.2f, useSecondary);
                if (sampleBehind > WaterY)
                    speedY += sampleBehind - WaterY;
                speedY += -gravity * WaterDiff * Time.deltaTime;
            }
            float nextY = transform.position.y + speedY * Time.deltaTime;
            if (AboveWater && nextY < WaterY)
            {
                OnHitWater(speedY);
                speedY *= 0.549f;
                nextY = transform.position.y + speedY * Time.deltaTime;
            }
            float clampedY = Mathf.Clamp(nextY, -12.0f, 12.0f);
            transform.position = new Vector3(transform.position.x, clampedY, transform.position.z);
        }
        else
        {
            transform.position = new Vector3(transform.position.x, WaterY, transform.position.z);
        }
        var targetRotation = Quaternion.LookRotation(Vector3.forward, wavesSum.primaryWave.NormalEval(transform.position.x)) * Quaternion.Euler(0.0f, 0.0f, rotationOffset);
        transform.rotation = Quaternion.Slerp(transform.rotation, targetRotation, normalRotationSmooth*Time.deltaTime) ;
    }

    //private void HitWater(float hitSpeed)
    //{
    //    if (hitSpeed > 0.0f)
    //        Debug.LogFormat("Splashed {0}", hitSpeed);
    //}

    private void OnDrawGizmos()
    {
        Gizmos.DrawWireSphere(transform.position, 0.2f);
    }
}
