﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Wave : MonoBehaviour {

    public float waveOffset = 0.0f;
    public float normalizedWaveOffset;
    public float amplitude = 1.0f;
    public float normalizedWaveLength = 1.0f;
    public const float normalizedCrestOffset = 0.25f;
    public const float normalizedValleyOffset = 0.75f;
    public const float viewSize = 20.0f;
    public bool clamped = false;
    public Matrix4x4 worldToLocalWaveMatrix {  get { return Matrix4x4.TRS(Vector2.right * normalizedWaveOffset, Quaternion.identity, new Vector3(normalizedWaveLength, amplitude)); } }
    private WavesSum wavesSum;
    public Material waveMaterial;
    public void OnEnable()
    {
        wavesSum = GameObject.FindGameObjectWithTag("WavesSum").GetComponent<WavesSum>();
        if(wavesSum.primaryWave != this)
        {
            wavesSum.waves.Add(this);
            wavesSum.SetSecondary(this);
        }
    }
    public void OnDisable()
    {
        wavesSum.waves.Remove(this);
    }
    public Vector2 NormalEval(float worldX)
    {
        float waveX = WaveX(worldX);
        float tx = 1;
        float ty = Mathf.Cos(waveX) / Mathf.Sqrt(1 + Mathf.Cos(waveX)* Mathf.Cos(waveX));
        return new Vector2(ty, -tx*Mathf.Sign(amplitude));
    }
    public float WorldEval(float worldX)
    {
        return LocalEval((worldX - normalizedWaveOffset) / normalizedWaveLength);
    }
    public float LocalEval(float localX)
    {
        if(clamped)
            if (localX < 0.0f || localX > 1.0f)
                return 0.0f;
        float waveX = (localX)* Mathf.PI * 2;
        return (Mathf.Sin(waveX+ waveOffset)* amplitude);
    }
    public void Update()
    {
        if(waveMaterial != null)
        waveMaterial.SetVector("_WaveParams", new Vector4(normalizedWaveOffset, amplitude, normalizedWaveLength, waveOffset));
    }
    public float WaveX(float worldX)
    {
        return ((worldX - normalizedWaveOffset) / normalizedWaveLength) * Mathf.PI * 2 + waveOffset;
    }
    private void OnDrawGizmos()
    {
        //Ocean Line
        Gizmos.color = Color.red;
        Gizmos.DrawLine(-Vector2.right * 0.0f, Vector2.right * viewSize);

        Gizmos.matrix = worldToLocalWaveMatrix;
        Gizmos.color = new Color(0.0f, 1.0f, 0.0f, 1.0f);
        //Wave
        int segments = 150;
        Vector2 last = Vector2.zero;
        for (int i = 0; i <= segments; i++)
        {
            float normX = (i / (float)segments) * Mathf.PI * 2;
            float x = normX;
            if (!clamped)
                x *= (viewSize / normalizedWaveLength);
            float y = Mathf.Sin(x + waveOffset);
            Vector2 next = new Vector2(x/(Mathf.PI*2), y);
            Gizmos.DrawLine(last, next);
            last = next;
        }
        Gizmos.color = Color.white;
        Gizmos.matrix = Matrix4x4.identity;
        Gizmos.DrawLine(Vector2.zero, NormalEval(0.0f));
        Gizmos.matrix = Matrix4x4.identity;
    }
}
