﻿// Upgrade NOTE: replaced '_Object2World' with 'unity_ObjectToWorld'

Shader "Unlit/Water"
{
	Properties
	{
		_Color("Color", Color) = (1,1,1,1)
		_FoamTex("Foam Color", 2D) = "white" {}
		_WaveParams("Wave (Off, Amp, Length)", Vector) = (0, -0.6, 7, 0.0)
			_FoamSpeed("Foam Speed", Float) = 2.0
			_FoamScale("FoamScale", Float) = 2.0
	}
	SubShader
	{
		Tags
		{
			"Queue" = "Transparent"
			"IgnoreProjector" = "True"
			"RenderType" = "Transparent"
			"PreviewType" = "Plane"
			"CanUseSpriteAtlas" = "True"
		}

		Cull Off
		Lighting Off
		ZWrite Off
		Pass
		{
			Blend DstColor Zero
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
			};

			struct v2f
			{
				float4 vertex : SV_POSITION;
				float3 worldPos : TEXCOORD0;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.worldPos = mul(unity_ObjectToWorld, v.vertex);
				return o;
			}

			float4 _Color;
			float4 _WaveParams;
			
			fixed4 frag (v2f i) : SV_Target
			{
				float x = i.worldPos.x*3.1415*2*(1.0/ _WaveParams.z) - _Time.y;
				float y = sin(x + _WaveParams.x)*_WaveParams.y;
				float diff = i.worldPos.y - y;
				float mask = diff < 0 ? 1.0 : 0.0;
				float4 waterColor = _Color.rgba;
				/*float2 uv = frac(float2(x + 0.1 + _Time.y*_FoamSpeed, -abs(diff+0.2*_FoamScale)*2.5/ _FoamScale));
				float4 foamColor = tex2D(_FoamTex, uv);*/
				return lerp(1, waterColor*mask , mask-diff);
			}
			ENDCG
		}
		Pass
		{
			Blend SrcAlpha OneMinusSrcAlpha
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

				struct appdata
			{
				float4 vertex : POSITION;
			};

			struct v2f
			{
				float4 vertex : SV_POSITION;
				float3 worldPos : TEXCOORD0;
			};

			v2f vert(appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.worldPos = mul(unity_ObjectToWorld, v.vertex);
				return o;
			}

			sampler2D _FoamTex;
			float _FoamSpeed;
			float _FoamScale;
			float4 _WaveParams;

			fixed4 frag(v2f i) : SV_Target
			{
				float x = i.worldPos.x*3.1415 * 2 * (1.0 / _WaveParams.z) - _Time.y;
				float y = sin(x + _WaveParams.x)*_WaveParams.y;
				float diff = i.worldPos.y - y;
				float mask = diff < 0 ? 1.0 : 0.0;
				float2 uv = frac(float2(x + 0.1 + _Time.y*_FoamSpeed, -abs(diff + 0.2*_FoamScale)*2.5 / _FoamScale));
				float4 foamColor = tex2D(_FoamTex, uv);
				return foamColor*foamColor.a*(abs(diff) < 0.2*_FoamScale);
			}
			ENDCG
		}
	}
}
